/* Foldroid - Mail Folder Selection Tool
 * Copyright (C) 2000-2010 Stefan Westerfeld
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __FST_FAM_H__
#define __FST_FAM_H__

#include <fam.h>

#include <map>
#include <list>
#include <vector>
#include <string>

namespace Foldroid {

class FamObserver {
public:
  virtual void notify_file_changed (const std::string& filename) = 0;
};

class Fam {
private:
  static Fam *instance;

  FAMConnection                                    m_fam_connection;
  std::map<std::string, std::list<FamObserver *> > m_observers;
  std::map<std::string, FAMRequest>                m_requests;

public:
  Fam();
  ~Fam();

  void add_file (FamObserver *observer, const std::string& filename);
  void remove_file (FamObserver *observer);
  void check();

  static Fam *the();
};

};
#endif /* __FST_FAM_H__ */

/* vim:set ts=8 sw=2 sts=2: */
