/* Foldroid - Mail Folder Selection Tool
 * Copyright (C) 2000-2010 Stefan Westerfeld
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "fddfolder.h"
#include "fddfolderthread.h"

#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>


using namespace std;
using namespace Foldroid;

Folder::Folder (const string& filename, const string& label) :
  m_filename (filename),
  m_label (label),
  m_messages (0),
  m_new_messages (0),
  m_file_changed (true)
{
  Fam::the()->add_file (this, m_filename);
}

Folder::Folder (const Folder& folder)
{
  m_filename = folder.m_filename;
  m_label = folder.m_label;
  m_messages = folder.m_messages;
  m_new_messages = folder.m_new_messages;
  m_file_changed = folder.m_file_changed;

  Fam::the()->add_file (this, m_filename);
}

Folder&
Folder::operator= (const Folder& folder)
{
  if (&folder != this)
    {
      m_filename = folder.m_filename;
      m_label = folder.m_label;
      m_messages = folder.m_messages;
      m_new_messages = folder.m_new_messages;
      m_file_changed = folder.m_file_changed;

      Fam::the()->add_file (this, m_filename);
    }
}

Folder::~Folder()
{
  Fam::the()->remove_file (this);
}

bool
Folder::need_update()
{
  return m_file_changed;
}

void
Folder::update()
{
  if (!need_update())
    return;

  FolderThread::the()->request_scan (m_filename);
  m_file_changed = false;
}

void
Folder::set_scan_result (int messages, int new_messages)
{
  m_messages = messages;
  m_new_messages = new_messages;
}

string
Folder::filename() const
{
  return m_filename;
}

string
Folder::description() const
{
  char buffer[1024];
  sprintf (buffer, "Msgs:%d New:%d", m_messages, m_new_messages);
  return buffer;
}

string
Folder::label() const
{
  return m_label;
}

void
Folder::notify_file_changed (const std::string&)
{
  m_file_changed = true;
}

/* vim:set ts=8 sw=2 sts=2: */
