/* Foldroid - Mail Folder Selection Tool
 * Copyright (C) 2010 Stefan Westerfeld
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef __FOLDROID_FOLDER_THREAD_H__
#define __FOLDROID_FOLDER_THREAD_H__

#include <vector>
#include <string>
#include <pthread.h>

namespace Foldroid {

class FolderThread
{
public:
  enum CommandType {
    COMMAND_STOP,
    COMMAND_SCAN
  };

  struct Command
  {
    CommandType command;
    std::string filename;
  };

  struct CommandResult
  {
    CommandType command;
    std::string filename;
    int         messages;
    int         new_flag;
  };

  void run();
  void request_scan (const std::string& filename);
  bool get_scan_result (CommandResult& result);

  static FolderThread* the();
  static void start();
  static void stop();

private:
  pthread_mutex_t            m_folder_thread_mutex;

  std::vector<Command>       m_command_queue;
  std::vector<CommandResult> m_result_queue;

  void scan (const std::string& filename);

};

}
#endif /* __FST_FOLDER_THREAD_H__ */
