/* Foldroid - Mail Folder Selection Tool
 * Copyright (C) 2000-2010 Stefan Westerfeld
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "fddscan.h"
#include "foldroid.h"
#include <assert.h>
#include <stdlib.h>
#include <map>

using namespace Foldroid;
using namespace std;

Scan::Scan() :
  m_config (UserInterface::default_config()),
  m_changes (0)
{
  //
}

void
Scan::update_folders()
{
  const int fsHave = 1;
  const int fsNeed = 2;
  const int fsGood = 3;

  map<string, int> folderstate;
  map<string, string> folderlabel;

  /* mark folders we currently have status information for with fsHave */
  list<Folder>::iterator i;
  for (i = m_folders.begin(); i != m_folders.end(); i++)
    {
      folderstate[i->filename()] |= fsHave;
      folderlabel[i->filename()] = i->label();
    }

  /* mark folders we need status information for with fsNeed */
  vector<FolderConfig> folderfiles = m_config.folders();
  vector<FolderConfig>::iterator ffi;
  for (ffi = folderfiles.begin(); ffi != folderfiles.end(); ffi++)
    {
      folderstate[ffi->filename] |= fsNeed;
      folderlabel[ffi->filename] = ffi->label;
    }

  /* handle cases where not both flags are set */
  map<string,int>::iterator fsi;
  for (fsi = folderstate.begin(); fsi != folderstate.end(); fsi++)
    {
      if (fsi->second != fsGood)
        {
          if (fsi->second == fsHave)
            {
              /* we don't need that folder anymore */
              i = m_folders.begin();
              while (i != m_folders.end())
                {
                  if (i->filename() == fsi->first)
                    i = m_folders.erase (i);
                  else
                    i++;
                }
            }
          else if (fsi->second == fsNeed)
            {
              m_folders.push_back (Folder (fsi->first, folderlabel[fsi->first]));
            }
          else
            {
              assert (false); /* what else? */
            }
        }
    }
}


bool
Scan::need_update()
{
  FolderThread *folder_thread = FolderThread::the();

  FolderThread::CommandResult result;
  while (folder_thread->get_scan_result (result))
    {
      list<Folder>::iterator i;
      for (i = m_folders.begin(); i != m_folders.end(); i++)
        {
          if (i->filename() == result.filename)
            {
              i->set_scan_result (result.messages, result.new_flag);
              m_changes++;
            }
        }
    }
  update_folders();

  m_fam.check();

  list<Folder>::iterator i;
  for (i = m_folders.begin(); i != m_folders.end(); i++)
    if (i->need_update())
      return true;
  return false;
}

void Scan::update (UserInterface& ui)
{
  update_folders();

  size_t num = 0;
  size_t count = m_folders.size();

  list<Folder>::iterator i;
  for (i = m_folders.begin(); i != m_folders.end(); i++)
    {
      char buffer[1024];
      sprintf (buffer, "updating folder %u/%u: %20s", ++num, count, i->label().c_str());
      ui.print_status (buffer);
      i->update();
    }
}

static
bool folder_label_cmp (const Folder& f1, const Folder& f2)
{
  return f1.label() < f2.label();
}

const list<Folder>&
Scan::folders()
{
  m_folders.sort (folder_label_cmp);
  return m_folders;
}

int
Scan::changes()
{
  return m_changes;
}

/* vim:set ts=8 sw=2 sts=2: */
